@@ -1,14 +1,18 @@
+use thiserror::Error;
 use ethabi::Token;
+use std::process::Command;
 use std::collections::HashMap;
 use std::time::{Duration, Instant};
 use serde_json::{Value as JSONValue, json};
 
-use crate::abi::ABI;
+use crate::tassl::TASSLError;
+use crate::channel::ChannelError;
 use crate::config::Config;
-use crate::transaction::get_sign_transaction_data;
-use crate::account::{Account, create_account_from_pem};
-use crate::web3::{fetcher_trait::FetcherTrait, service_error::ServiceError};
+use crate::abi::{ABI, ABIError};
+use crate::transaction::{TransactionError, get_sign_transaction_data};
+use crate::account::{Account, AccountError, create_account_from_pem};
 use crate::helpers::{parse_json_string, parse_json_string_array, convert_hex_str_to_u32};
+use crate::web3::{fetcher_trait::FetcherTrait, rpc_fetcher::RPCFetcher, channel_fetcher::ChannelFetcher};
 
 fn generate_request_params(method: &str, params: &JSONValue) -> JSONValue {
     json!({
@@ -19,6 +23,47 @@ fn generate_request_params(method: &str, params: &JSONValue) -> JSONValue {
     })
 }
 
+#[derive(Error, Debug)]
+pub enum ServiceError {
+    #[error("std::io::Error")]
+    StdIOError(#[from] std::io::Error),
+
+    #[error("hyper::Error")]
+    HyperError(#[from] hyper::Error),
+
+    #[error("hyper::http::Error")]
+    HyperHttpError(#[from] hyper::http::Error),
+
+    #[error("serde_json::Error")]
+    SerdeJsonError(#[from] serde_json::Error),
+
+    #[error("channel error")]
+    ChannelError(#[from] ChannelError),
+
+    #[error("tassl error")]
+    TASSLError(#[from] TASSLError),
+
+    #[error("abi error")]
+    ABIError(#[from] ABIError),
+
+    #[error("account error")]
+    AccountError(#[from] AccountError),
+
+    #[error("transaction error")]
+    TransactionError(#[from] TransactionError),
+
+    #[error("fisco bcos custom error")]
+    CustomError {
+        message: String,
+    },
+
+    #[error("fisco bcos response error")]
+    FiscoBcosError {
+        code: i32,
+        message: String,
+    }
+}
+
 #[derive(Debug)]
 pub struct CallResponse {
     pub current_block_number: String,
@@ -27,14 +72,82 @@ pub struct CallResponse {
 }
 
 pub struct Service {
-    pub config: Config,
+    config: Config,
     account: Account,
     fetcher: Box<dyn FetcherTrait + Send + Sync>,
 }
 
 impl Service {
     fn get_abi(&self, contract_name: &str) -> Result<ABI, ServiceError> {
-        Ok(ABI::new(&self.config.contract, contract_name, self.config.sm_crypto)?)
+        Ok(ABI::new_with_contract_config(&self.config.contract, contract_name, self.config.sm_crypto)?)
+    }
+
+    pub(crate) async fn send_transaction_with_abi(
+        &self,
+        method: &str,
+        to_address: &str,
+        abi: &ABI,
+        function_name: &str,
+        tokens: &Vec<Token>,
+    ) -> Result<String, ServiceError> {
+        let block_number = convert_hex_str_to_u32(&self.get_block_number().await?);
+        let data = abi.encode_function_input(function_name, tokens)?;
+        let transaction_data = get_sign_transaction_data(
+            &self.account,
+            self.config.group_id,
+            self.config.chain_id,
+            block_number + 500,
+            to_address,
+            &data,
+            self.config.sm_crypto,
+        )?;
+        let params = generate_request_params(
+            method,
+            &json!([self.config.group_id, format!("0x{}", hex::encode(&transaction_data))]),
+        );
+        Ok(parse_json_string(&self.fetcher.fetch(&params).await?))
+    }
+
+    pub(crate) async fn call_with_abi(
+        &self,
+        to_address: &str,
+        abi: &ABI,
+        function_name: &str,
+        tokens: &Vec<Token>,
+    ) -> Result<CallResponse, ServiceError> {
+        let transaction_data = abi.encode_function_input(function_name, tokens)?;
+        let params = json!({
+            "from": format!("0x{}", hex::encode(&self.account.address)),
+            "to": to_address.to_owned(),
+            "value": "0x0",
+            "data": format!("0x{}", hex::encode(&transaction_data)),
+        });
+        let response = self.fetcher.fetch(
+            &generate_request_params("call", &json!([self.config.group_id, params]))
+        ).await?;
+        Ok(CallResponse {
+            status: parse_json_string(&response["status"]),
+            current_block_number: parse_json_string(&response["currentBlockNumber"]),
+            output: abi.decode_output(function_name, &parse_json_string(&response["output"]))?,
+        })
+    }
+
+    pub(crate) async fn get_transaction_receipt_with_timeout(
+        &self,
+        transaction_hash: &str,
+    ) -> Result<JSONValue, ServiceError> {
+        let start = Instant::now();
+        let timeout_milliseconds = (1000 * self.config.timeout_seconds) as u128;
+        while Instant::now().duration_since(start).as_millis() < timeout_milliseconds {
+            let transaction_receipt: JSONValue = self.get_transaction_receipt(transaction_hash).await?;
+            if transaction_receipt.is_null() {
+                tokio::time::sleep(Duration::from_millis(200)).await;
+                continue;
+            }
+            let transaction_receipt = self.get_transaction_receipt(transaction_hash).await?;
+            return Ok(transaction_receipt);
+        }
+        Ok(json!(null))
     }
 
     pub fn new(config: &Config, fetcher: Box<dyn FetcherTrait + Send + Sync>) -> Result<Service, ServiceError> {
@@ -47,6 +160,10 @@ impl Service {
         )
     }
 
+    pub fn get_config(&self) -> Config {
+        self.config.clone()
+    }
+
     pub async fn get_client_version(&self)  -> Result<JSONValue, ServiceError> {
         let params = generate_request_params("getClientVersion", &json!([self.config.group_id]));
         Ok(self.fetcher.fetch(&params).await?)
@@ -234,48 +351,7 @@ impl Service {
         tokens: &Vec<Token>,
     ) -> Result<CallResponse, ServiceError> {
         let abi = self.get_abi(contract_name)?;
-        let transaction_data = abi.encode_function_input(function_name, tokens)?;
-        let params = json!({
-            "from": format!("0x{}", hex::encode(&self.account.address)),
-            "to": to_address.to_owned(),
-            "value": "0x0",
-            "data": format!("0x{}", hex::encode(&transaction_data)),
-        });
-        let response = self.fetcher.fetch(
-            &generate_request_params("call", &json!([self.config.group_id, params]))
-        ).await?;
-        Ok(CallResponse {
-            status: parse_json_string(&response["status"]),
-            current_block_number: parse_json_string(&response["currentBlockNumber"]),
-            output: abi.decode_output(function_name, &parse_json_string(&response["output"]))?,
-        })
-    }
-
-    async fn send_transaction(
-        &self,
-        method: &str,
-        contract_name: &str,
-        to_address: &str,
-        function_name: &str,
-        tokens: &Vec<Token>,
-    ) -> Result<String, ServiceError> {
-        let block_number = convert_hex_str_to_u32(&self.get_block_number().await?);
-        let abi = self.get_abi(contract_name)?;
-        let data = abi.encode_function_input(function_name, tokens)?;
-        let transaction_data = get_sign_transaction_data(
-            &self.account,
-            self.config.group_id,
-            self.config.chain_id,
-            block_number + 500,
-            to_address,
-            &data,
-            self.config.sm_crypto,
-        )?;
-        let params = generate_request_params(
-            method,
-            &json!([self.config.group_id, format!("0x{}", hex::encode(&transaction_data))]),
-        );
-        Ok(parse_json_string(&self.fetcher.fetch(&params).await?))
+        Ok(self.call_with_abi(to_address, &abi, function_name, tokens).await?)
     }
 
     pub async fn send_raw_transaction(
@@ -285,7 +361,8 @@ impl Service {
         function_name: &str,
         tokens: &Vec<Token>,
     ) -> Result<String, ServiceError> {
-        Ok(self.send_transaction("sendRawTransaction", contract_name, to_address, function_name, tokens).await?)
+        let abi = self.get_abi(contract_name)?;
+        Ok(self.send_transaction_with_abi("sendRawTransaction", to_address, &abi, function_name, tokens).await?)
     }
 
     pub async fn send_raw_transaction_and_get_proof(
@@ -295,7 +372,8 @@ impl Service {
         function_name: &str,
         tokens: &Vec<Token>,
     ) -> Result<String, ServiceError> {
-        Ok(self.send_transaction("sendRawTransactionAndGetProof", contract_name, to_address, function_name, tokens).await?)
+        let abi = self.get_abi(contract_name)?;
+        Ok(self.send_transaction_with_abi("sendRawTransactionAndGetProof", to_address, &abi, function_name, tokens).await?)
     }
 
     pub async fn deploy(&self, contract_name: &str, tokens: &Vec<Token>) -> Result<JSONValue, ServiceError> {
@@ -316,28 +394,21 @@ impl Service {
             &json!([self.config.group_id, format!("0x{}", hex::encode(&transaction_data))]),
         );
         let transaction_hash = parse_json_string(&self.fetcher.fetch(&params).await?);
-        let start = Instant::now();
-        let timeout_milliseconds = (1000 * self.config.timeout_seconds) as u128;
-        while Instant::now().duration_since(start).as_millis() < timeout_milliseconds {
-            let transaction_receipt: JSONValue = self.get_transaction_receipt(&transaction_hash).await?;
-            if transaction_receipt.is_null() {
-                tokio::time::sleep(Duration::from_millis(200)).await;
-                continue;
-            }
-            let transaction_receipt = self.get_transaction_receipt(&transaction_hash).await?;
-            return Ok(json!({
+        let transaction_receipt = self.get_transaction_receipt_with_timeout(&transaction_hash).await?;
+        if transaction_receipt.is_null() {
+            Err(ServiceError::CustomError {
+                message: format!(
+                    "Contract deployed, but the action for fetching transaction receipt is timeout. Transaction hash is {:?}",
+                    transaction_hash
+                ),
+            })
+        } else {
+            Ok(json!({
                 "status": transaction_receipt["status"],
                 "transactionHash": transaction_receipt["transactionHash"],
                 "contractAddress": transaction_receipt["contractAddress"]
-            }));
+            }))
         }
-        Err(ServiceError::FiscoBcosError {
-            code: -1,
-            message: format!(
-                "Contract deployed, but the action for fetching transaction receipt is timeout. Transaction hash is {:?}",
-                transaction_hash
-            ),
-        })
     }
 
     ///
@@ -348,8 +419,43 @@ impl Service {
         contract_name: &str,
         link_libraries: &Option<HashMap<String, String>>,
     ) -> Result<(), ServiceError> {
-        let mut abi = self.get_abi(contract_name)?;
-        Ok(abi.compile(link_libraries)?)
+        let contract_path = self.config.contract.get_contract_path(contract_name);
+        if !contract_path.is_file() {
+            return Err(
+                ServiceError::CustomError {
+                    message: format!("Can't find the contract:{:}", contract_name)
+                }
+            );
+        }
+
+        let mut compile_command = Command::new(&self.config.contract.solc);
+        let link_libraries: Vec<String> = match link_libraries {
+            None => vec![],
+            Some(link_libraries) => {
+                let mut result: Vec<String> = vec![];
+                for (name, address) in link_libraries  {
+                    result.push(format!("{:}.sol:{:}:{:}", contract_name, name, address))
+                }
+                result
+            }
+        };
+        if link_libraries.len() > 0 {
+            compile_command.arg("--libraries");
+            compile_command.arg(link_libraries.join(" "));
+        }
+        compile_command.arg("--overwrite").arg("--abi").arg("--bin").arg("-o");
+        compile_command.arg(self.config.contract.output.clone());
+        compile_command.arg(contract_path);
+        let status = compile_command.status()?;
+        if status.success() {
+            Ok(())
+        } else {
+            Err(
+                ServiceError::CustomError {
+                    message: format!("Can't compile the contract:{:}, please try it again later", contract_name)
+                }
+            )
+        }
     }
 
     pub async fn get_transaction_by_hash_with_proof(&self, transaction_hash: &str) -> Result<JSONValue, ServiceError> {
@@ -454,4 +560,15 @@ impl Service {
         );
         Ok(self.fetcher.fetch(&params).await?)
     }
+}
+
+pub fn create_service_with_config(config: &Config) -> Result<Service, ServiceError>  {
+    if config.service_type.eq("rpc") {
+        let fetcher = RPCFetcher::new(&config.node.host, config.node.port);
+        Service::new(&config, Box::new(fetcher)
+        )
+    } else {
+        let fetcher = ChannelFetcher::new(&config);
+        Service::new(&config, Box::new(fetcher))
+    }
 }
\ No newline at end of file
